/*
	$Id: SCContextMenuItemCell.java,v 1.2 2002/12/08 16:10:04 phonohawk Exp $
	----------
	
    "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
    Copyright (C) 2001 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

public class SCContextMenuItemCell extends NSMenuItemCell
{
    public static final int ALIGN_LEFT = 1;
    public static final int ALIGN_RIGHT = 2;
    
    SCSession session;
    int type; // SCFoundation.HONTAI || SCFoundation.UNYUU
    int text_alignment;
    SCMenuAppearanceServer mas;
    
    public SCContextMenuItemCell(SCSession session,int type)
    {
		super();
		this.session = session;
		this.type = type;
		text_alignment = ALIGN_LEFT;
		
		mas = session.getMenuAppearance();
    }
    
    public SCContextMenuItemCell(SCSession session,int type,int text_alignment)
    {
		super();
		this.session = session;
		this.type = type;
		this.text_alignment = text_alignment;
		
		mas = session.getMenuAppearance();
    }
    
    public NSSize cellSize()
    {
		NSSize size = super.cellSize();
		if (image() == null)
		{
			return size;
		}
		else
		{
			if (image().size().height() > size.height())
			{
				return new NSSize(size.width(),image().size().height());
			}
			else
			{
				return size;
			}
		}
    }
    
    NSMutableRect _r = null;
    public void drawWithFrameInView(NSRect r,NSView view)
    {
		if (!mas.hasImage())
		{
			super.drawBorderAndBackgroundWithFrameInView(r,view);
			if (menuItem().isSeparatorItem())
			{
				super.drawSeparatorItemWithFrameInView(r,view);
			}
			else
			{
				super.drawImageWithFrameInView(r,view);
				super.drawKeyEquivalentWithFrameInView(r,view);
				super.drawStateImageWithFrameInView(r,view);
				super.drawTitleWithFrameInView(r,view);
			}
		}
		else
		{
			if (_r == null)
			{
				_r = new NSMutableRect();
			}
			_r.setOrigin(r.origin());
			_r.setSize(r.size());
			_r.setX(_r.x() + ((SCContextMenuView)menuView()).getSidebarWidth());
			_r.setWidth(_r.width() - ((SCContextMenuView)menuView()).getSidebarWidth());
			
			drawBorderAndBackgroundWithFrameInView(_r,view);
			if (menuItem().isSeparatorItem())
			{
				drawSeparatorItemWithFrameInView(_r,view);
			}
			else
			{
				drawImageWithFrameInView(_r,view);
				drawKeyEquivalentWithFrameInView(_r,view);
				drawStateImageWithFrameInView(_r,view);
				drawTitleWithFrameInView(_r,view);
			}
		}
    }
    
    public void drawBorderAndBackgroundWithFrameInView(NSRect r,NSView view)
    {
		if (!isHighlighted()) return;
		
		NSImage image = (isHighlighted() ? mas.getFgImage() : mas.getBgImage());
		int alignment = (isHighlighted() ? mas.getAlignmentOfFg() : mas.getAlignmentOfBg());
		NSColor bottomRightColor = (isHighlighted() ? mas.getBottomRightColorOfFgImage() : mas.getBottomRightColorOfBgImage());
		NSColor topLeftColor = (isHighlighted() ? mas.getTopLeftColorOfFgImage() : mas.getTopLeftColorOfBgImage());
		
		float widthOfView = view.frame().width();
		float heightOfView = view.frame().height();
		float widthOfImage = image.size().width();
		float heightOfImage = image.size().height();
		
		switch (alignment)
		{
			case	SCMenuAppearanceServer.ALIGN_LEFTTOP:
			{
				float destX = r.x();
				float destY = r.y();
				float srcWidth = (widthOfImage > r.width() ? r.width() : widthOfImage);
				float srcHeight = (destY + r.height() > heightOfImage ? destY + r.height() - heightOfImage : r.height());
				if (view.isFlipped() && srcHeight < r.height())
				{
					destY += r.height() - srcHeight;
				}
		
				if (srcHeight > 0)
				{
					NSPoint destPoint = new NSPoint(destX,(view.isFlipped() ? destY + srcHeight : destY));
					
					float srcX = 0;
					float srcY = (view.isFlipped() ? heightOfImage - destY - srcHeight : r.y() - (heightOfView - heightOfImage));
					NSRect srcRect = new NSRect(srcX,srcY,srcWidth,srcHeight);
					
					image.compositeToPointFromRect(destPoint,srcRect,NSImage.CompositeCopy);
				}
				
				// E̗]
				if (r.width() > widthOfImage)
				{
					NSRect rectToFill = new NSRect(r.x() + srcWidth,
								r.y(),
								r.width() - widthOfImage,
								r.height());
					bottomRightColor.set();
					NSBezierPath.fillRect(rectToFill);
				}
				
				// ̗]
				if (destY + r.height() > heightOfImage)
				{
					NSRect rectToFill = new NSRect(r.x(),
								destY,
								r.width(),
								r.y() + r.height() - destY);
					bottomRightColor.set();
					NSBezierPath.fillRect(rectToFill);
				}
			}
			break;
		
			case	SCMenuAppearanceServer.ALIGN_RIGHTTOP:
			{
				float destX = (widthOfImage > r.width() ? r.x() : r.x() + (r.width() - widthOfImage));
				float destY = r.y();
				float srcWidth = (widthOfImage > r.width() ? r.width() : widthOfImage);
				float srcHeight = (destY + r.height() > heightOfImage ? destY + r.height() - heightOfImage : r.height());
				if (view.isFlipped() && srcHeight < r.height())
				{
					destY += r.height() - srcHeight;
				}
				
				if (srcHeight > 0)
				{
					NSPoint destPoint = new NSPoint(destX,(view.isFlipped() ? destY + srcHeight : destY));
					
					float srcX = (destX > r.x() ? 0 : widthOfImage - r.width());
					float srcY = (view.isFlipped() ? heightOfImage - destY - srcHeight : r.y() - (heightOfView - heightOfImage));
					NSRect srcRect = new NSRect(srcX,srcY,srcWidth,srcHeight);
					
					image.compositeToPointFromRect(destPoint,srcRect,NSImage.CompositeCopy);
				}
		
				// ̗]
				if (r.width() > widthOfImage)
				{
					NSRect rectToFill = new NSRect(r.x(),
								r.y(),
								r.width() - widthOfImage,
								r.height());
					bottomRightColor.set();
					NSBezierPath.fillRect(rectToFill);
				}
				
				// ̗]
				if (destY + r.height() > heightOfImage)
				{
					NSRect rectToFill = new NSRect(r.x(),
								destY,
								r.width(),
								r.y() + r.height() - destY);
					bottomRightColor.set();
					NSBezierPath.fillRect(rectToFill); 
				}
			}
			break;
			
			case	SCMenuAppearanceServer.ALIGN_LEFTBOTTOM:
			{
				float destX = r.x();
				float destY = r.y();
				float srcWidth = (widthOfImage > r.width() ? r.width() : widthOfImage);
				float srcHeight = (destY < heightOfView - heightOfImage ? heightOfView - heightOfImage - destY : r.height());
				
				if (srcHeight > 0)
				{
					NSPoint destPoint = new NSPoint(destX,destY + srcHeight);
					
					float srcX = 0;
					float srcY = heightOfView - (destY + srcHeight);
					NSRect srcRect = new NSRect(srcX,srcY,srcWidth,srcHeight);
					
					image.compositeToPointFromRect(destPoint,srcRect,NSImage.CompositeCopy);
				}
				
				// E̗]
				if (r.width() > widthOfImage)
				{
					NSRect rectToFill = new NSRect(destX + srcWidth,
								r.y(),
								r.width() - widthOfImage,
								r.height());
					topLeftColor.set();
					NSBezierPath.fillRect(rectToFill);
				}
				
				// ̗]
				if (destY < heightOfView - heightOfImage)
				{
					NSRect rectToFill = new NSRect(destX,
								destY,
								r.width(),
								r.y() + r.height() - destY);
					topLeftColor.set();
					NSBezierPath.fillRect(rectToFill); 
				}
			}
			break;
			
			case	SCMenuAppearanceServer.ALIGN_RIGHTBOTTOM:
			{
				float destX = (widthOfImage > r.width() ? r.x() : r.x() + (r.width() - widthOfImage));
				float destY = r.y();
				float srcWidth = (widthOfImage > r.width() ? r.width() : widthOfImage);
				float srcHeight = (destY < heightOfView - heightOfImage ? heightOfView - heightOfImage - destY : r.height());
				
				if (srcHeight > 0)
				{
					NSPoint destPoint = new NSPoint(destX,destY + srcHeight);
					
					float srcX = (destX > r.x() ? 0 : widthOfImage - r.width());
					float srcY = heightOfView - (destY + srcHeight);
					NSRect srcRect = new NSRect(srcX,srcY,srcWidth,srcHeight);
					
					image.compositeToPointFromRect(destPoint,srcRect,NSImage.CompositeCopy);
				}
				
				// ̗]
				if (r.width() > widthOfImage)
				{
					NSRect rectToFill = new NSRect(r.x(),
								r.y(),
								r.width() - widthOfImage,
								r.height());
					bottomRightColor.set();
					NSBezierPath.fillRect(rectToFill);
				}
				
				// ̗]
				if (destY < heightOfView - heightOfImage)
				{
					NSRect rectToFill = new NSRect(r.x(),
								destY,
								r.width(),
								r.y() + r.height() - destY);
					topLeftColor.set();
					NSBezierPath.fillRect(rectToFill); 
				}
			}
			break;
		}
    }
    
    public void drawTitleWithFrameInView(NSRect r,NSView view)
    {
		NSColor color = (isHighlighted() ? mas.getFgFontColor() : (menuItem().isEnabled() ? mas.getBgFontColor() : mas.getDisabledColor()));
		
		NSMutableAttributedString astr = new NSMutableAttributedString(title());
		NSRange range_whole = new NSRange(0,title().length());
		astr.addAttributeInRange(NSAttributedString.FontAttributeName,font(),range_whole);
		astr.addAttributeInRange(NSAttributedString.ForegroundColorAttributeName,color,range_whole);
		NSSize strSize = NSGraphics.sizeOfAttributedString(astr);
		
		if (text_alignment == ALIGN_LEFT)
		{
			NSGraphics.drawAttributedString(astr,titleRectForBounds(r).origin());
		}
		else
		{
			NSPoint destPoint = new NSPoint(menuView().keyEquivalentOffset() - 5 - strSize.width(),
							r.y() + (r.height() / 2.0f) - (strSize.height() / 2.0f));
			NSGraphics.drawAttributedString(astr,destPoint);
		}
    }
    
    public void drawSeparatorItemWithFrameInView(NSRect r,NSView view)
    {
		mas.getSeparatorColor().set();
		NSBezierPath.setDefaultLineWidth(1.0f);
		NSBezierPath.setDefaultLineCapStyle(NSBezierPath.LineCapStyleButt);
		NSBezierPath.strokeLineFromPoint(new NSPoint(r.x(),r.y() + (r.height() / 2.0f)),new NSPoint(r.maxX(),r.y() + (r.height() / 2.0f)));
    }
    
    public void drawKeyEquivalentWithFrameInView(NSRect r,NSView view)
    {
		if (!menuItem().hasSubmenu()) return;
		
		NSColor color = (isHighlighted() ? mas.getFgFontColor() : (menuItem().isEnabled() ? mas.getBgFontColor() : mas.getDisabledColor()));
		
		NSMutableAttributedString astr = new NSMutableAttributedString("\u25b6");
		NSRange range_whole = new NSRange(0,1);
		astr.addAttributeInRange(NSAttributedString.FontAttributeName,NSFont.systemFontOfSize(10.0f),range_whole);
		astr.addAttributeInRange(NSAttributedString.ForegroundColorAttributeName,color,range_whole);
		//NSSize strSize = NSGraphics.sizeOfAttributedString(astr);
		
		NSGraphics.drawAttributedString(astr,keyEquivalentRectForBounds(r).origin());
    }
}
