/*  $Id: SCGhostManager.java,v 1.6 2004/01/02 11:28:41 phonohawk Exp $
	
    "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
    Copyright (C) 2001-2003 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;
import java.util.*;

public class SCGhostManager {
    private static SCGhostManager _sharedInstance = null;
    
    SCGhostManagerWindowController managerWindowController;
    SCInstalledGhostsList installedList;
    SCInstalledBalloonsList balloonsList;
    SCShellsList shellsList;
    
    public static SCGhostManager sharedGhostManager() {
        if (_sharedInstance == null) {
            _sharedInstance = new SCGhostManager();
        }
        return _sharedInstance;
    }
    
    public SCGhostManager() {
        managerWindowController = new SCGhostManagerWindowController(this);
        
        balloonsList = new SCInstalledBalloonsList(this);
	shellsList = new SCShellsList(this);
	installedList = new SCInstalledGhostsList(this);
	
	NSNotificationCenter.defaultCenter().addObserver(
	    this,
	    new NSSelector("notification_reload",new Class[] {NSNotification.class}),
	    "installer.end",
	    null);
    }
    
    public void reloadLists() {
	installedList.reloadList();
	balloonsList.reloadList();
    }
    
    public void notification_reload(NSNotification notification) {
	SCOldTypeConverter.convertAll();
	SCOldBalloonConverter.convertAll();
	SCGhostThumbnailMover.moveAll();
	reloadLists();
    }
    
    public void showManagerWindow() {
        managerWindowController.showWindow(null);
    }
    
    public SCGhostManagerWindowController getWindowController() {
        return managerWindowController;
    }
    
    public SCInstalledGhostsList getInstalledList() {
        return installedList;
    }
    
    public SCInstalledBalloonsList getBalloonsList() {
        return balloonsList;
    }
    
    public SCShellsList getShellsList() {
	return shellsList;
    }
    
    public void bootAllGhostsToBoot() {
        // SCInstalledGhostsList.bootAllGhostsToBootɂ܂B
        installedList.bootAllGhostsToBoot();
	//(new _ZZZ()).start();
    }

    public void clearBootFlags() {
	installedList.clearBootFlags();
    }
    
    /*private class _ZZZ extends Thread
	{
		public void run()
		{
			int pool = NSAutoreleasePool.push();
			installedList.bootAllGhostsToBoot();
			NSAutoreleasePool.pop(pool);
		}
    }*/
    
    public Vector getAllGhostPathsInAsleep()
    {
		return installedList.getAllGhostPathsInAsleep();
    }
    
    public String getBalloonPathOfGhost(String ghostPath)
    {
		return installedList.getBalloonPathOfGhost(ghostPath);
    }
    
    public double getScaleOfGhost(String ghostPath)
    {
		return installedList.getScaleOfGhost(ghostPath);
    }
    
    public String getNameOfGhost(String ghostPath)
    {
		return installedList.getNameOfGhost(ghostPath);
    }
    
    public void forceHaltGhost(SCSession session)
    {
		// Ԃt@CɕۑăZbV܂A
		// ̍ۂOnClose𔭍s܂B
		if (session.isInPassiveMode()) return;
		
		getInstalledList().setBootFlag(session.getGhostPath(),false);
		setBootFlagInDefaults(session.getGhostPath(),false);
		session.close();
    }
    
    public void closeGhost(SCSession session)
    {
		// Ԃt@CɕۑăZbV܂B
		if (session.isInPassiveMode()) return;
		
		getInstalledList().setBootFlag(session.getGhostPath(),false);
		setBootFlagInDefaults(session.getGhostPath(),false);
		session.performClose();
    }
    
    public NSMutableDictionary getGhostDefaults(String path)
    {
        // path : home/ghost/[^/]+
        // ȂĒr[ȐK\ŏĂ݂肷B
        NSUserDefaults defaults = NSUserDefaults.standardUserDefaults();
        NSDictionary dic = defaults.dictionaryForKey("ghost.pref."+path);
        return (dic == null ? new NSMutableDictionary() : new NSMutableDictionary(dic));
    }
    
    public void setGhostDefaults(String path,NSDictionary dic)
    {
        // path : home/ghost/[^/]+
        NSUserDefaults defaults = NSUserDefaults.standardUserDefaults();
        defaults.setObjectForKey(dic,"ghost.pref."+path);
        defaults.synchronize();
    }
    
    public void setBootFlagInDefaults(String path,boolean bool)
    {
		NSMutableDictionary ghostDefaults = getGhostDefaults(path);
		ghostDefaults.setObjectForKey(new Integer(bool ? 1 : 0),"boot_flag");
		setGhostDefaults(path,ghostDefaults);
    }
	
	public Vector getInstalledGhostList() {
		return installedList.getInstalledGhostList();
	}
    
    public NSMenu makeGhostMenu(SCSession self)
    {
		// selfɃ`FbN}[NtԂŃj[܂B
		// ܂AselfȊŐÑS[Xg͊DF\܂B
		return installedList.makeGhostMenu(self);
    }
    
    public NSMenu makeShellMenu(SCSession self)
    {
		// self̃JgVFɃ`FbN}[NtԂŃj[܂B
		return shellsList.makeShellMenu(self);
    }
    
    public NSMenu makeBalloonMenu(SCSession self)
    {
		// self̃Jgo[Ƀ`FbN}[NtԂŃj[܂B
		return balloonsList.makeBalloonMenu(self);
    }
}
