/*
	$Id: SCHeadlineSession.java,v 1.3 2002/12/08 16:10:04 phonohawk Exp $
	----------
	
    "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
    Copyright (C) 2001 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;
import java.io.*;

public class SCHeadlineSession extends Thread
{
    SCSession session;
    SCHLSensorPlugin sensor;
    
    public SCHeadlineSession(SCSession session,SCHLSensorPlugin sensor)
    {
		this.session = session;
		this.sensor = sensor;
    }
    
    public void run()
    {
		if (session.isInPassiveMode()) return;
		final int pool = NSAutoreleasePool.push();
		
		final File hashDir = new File(SCFoundation.getParentDirOfBundle(),"home/_system/headline_hash");
		final File tempDir = new File(SCFoundation.getParentDirOfBundle(),"home/_system/headline_temp");
		
		// ܂̓_E[hB
		final SCHLSensorResponse respDescription = sensor.request("GET Description HEADLINE/1.0\r\nSender: "+SCFoundation.STRING_FOR_SENDER+"\r\n\r\n");
		final String sitename = respDescription.getHeader("Name");
		final String dataurl = respDescription.getHeader("Data-Location");
		final String siteurl = respDescription.getHeader("Site-Location");
		
		session.doShioriEvent("OnHeadlinesenseBegin",new String[] {sitename,siteurl});
		
		File tempfile = null;
		try
		{
			tempfile = File.createTempFile("head",".tmp",tempDir);
			SCFileDownloader.download(dataurl,tempfile,30*1000);
		}
		catch (Exception e)
		{
			session.doShioriEvent("OnHeadlinesenseFailure",new String[] {"can't download"});
			if (tempfile != null && tempfile.exists()) tempfile.delete();
			NSAutoreleasePool.pop(pool);
			return;
		}
	
		// t@CMD5Zo
		final String calculatedHash = SCFileUtils.getMD5AsString(tempfile);
		
		/*
			XV`FbN
			
			headline_hashɃvOC̃fBNgMD5t@CƂt@C݂A
			ɂ̒gƂ΂̃t@CMD5Ɠ
			XVƔfB
		*/
		final File hashfile = new File(hashDir,SCMiscUtils.getMD5AsString(sensor.getDirName()));
		if (hashfile.exists())
		{
			try
			{
				// ۑĂMD5擾
				BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(hashfile)));
				String savedHash = br.readLine();
				br.close();
				
				if (savedHash.equals(calculatedHash))
				{
					// XVB
					session.doShioriEvent("OnHeadlinesenseComplete",new String[] {"no update"});
					tempfile.delete();
					NSAutoreleasePool.pop(pool);
					return;
				}
			}
			catch (Exception e){}
		}
		
		// nbVۑ
		try
		{
			if (hashfile.exists()) hashfile.delete();
			
			OutputStreamWriter osw = new OutputStreamWriter(new FileOutputStream(hashfile));
			osw.write(calculatedHash,0,calculatedHash.length());
			osw.flush();
			osw.close();
		}
		catch (Exception e){}
		
		// vOCɑM
		try
		{
			StringBuffer reqHeader = new StringBuffer("GET Headline HEADLINE/1.0\r\n");
			reqHeader.append("Sender: "+SCFoundation.STRING_FOR_SENDER+"\r\n");
			reqHeader.append("Content-Length: "+tempfile.length()+"\r\n");
			reqHeader.append("\r\n"); // wb_
			
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			baos.write(reqHeader.toString().getBytes("UTF-8"));
			
			FileInputStream fis = new FileInputStream(tempfile);
			byte[] buf = new byte[512];
			while (true)
			{
				int count = fis.read(buf);
				if (count == -1) break; // eof
				
				baos.write(buf,0,count);
			} // f[^
	    
			// M
			SCHLSensorResponse headlineResp = sensor.request(new ByteArrayInputStream(baos.toByteArray()));
			baos.close();
			
			if (headlineResp.getStatusMessage().endsWith("200 OK"))
			{
				// XNvg쐬
				StringBuffer script = new StringBuffer("\\b2");
				BufferedReader br = new BufferedReader(new StringReader(headlineResp.getContent()));
				while (true)
				{
					String line = br.readLine();
					if (line == null) break;
					
					script.append(line+"\\n");
				}
				br.close();
				
				session.doShioriEvent("OnHeadlinesense.OnFind",new String[] {
						sitename,
						siteurl,
						"First and Last",
						script.toString()
						});
			}
			else
			{
				session.doShioriEvent("OnHeadlinesenseFailure",new String[] {"can't analyze"});
			}
		}
		catch (Exception e){}
		
		// ꎞt@C͍폜B
		tempfile.delete();
		
		NSAutoreleasePool.pop(pool);
    }
}
