/*
	$Id: SCOnlineInstaller.java,v 1.3 2002/12/08 16:10:05 phonohawk Exp $
	----------
	
    "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
    Copyright (C) 2001 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;
import java.io.*;

public class SCOnlineInstaller extends Thread
{
    private static final int TIMEOUT = 30 * 1000;
    
    String url;
    SCSession target;
    
    public SCOnlineInstaller(String url)
    {
		this.url = url;
		this.target = null;
    }
    
    public SCOnlineInstaller(String url,SCSession target)
    {
		this.url = url;
		this.target = target;
    }
    
    public void run()
    {
		int pool = NSAutoreleasePool.push();
		sendShioriEvent("OnURLDropping",new String[] {url});
		
		SCStatusWindowController stat = SCFoundation.sharedFoundation().getStatusCenter().newStatusWindow();
		stat.setTypeToText();
		stat.show();
		stat.texttype_print("Online installer started...\n");
		
		stat.texttype_println("url : %[color,orange]"+url+"%[color,default]");
		
		File outfile = new File(SCFoundation.sharedFoundation().getParentDirOfBundle(),"home/nar/"+NSPathUtilities.lastPathComponent(url));
		stat.texttype_println("file home/nar/"+outfile.getName()+" exists : "+(outfile.exists() ? "%[color,yellow]YES%[color,default]" : "NO"));
		if (outfile.exists())
		{
			outfile.delete();
			stat.texttype_println("%[color,yellow]deleted home/nar/"+outfile.getName()+"%[color,default]");
		}
		
		SCStatusWindowController prog_download = SCFoundation.sharedFoundation().getStatusCenter().newStatusWindow();
		prog_download.setTypeToProgressBar();
		prog_download.show();
		prog_download.pbartype_setText1("Downloading...");
		try
		{
			SCFileDownloader.download(url,outfile,TIMEOUT,prog_download);
		}
		catch (InterruptedIOException e)
		{
			sendShioriEvent("OnURLDropFailure");
			prog_download.pbartype_setText1("Timed Out.");
			prog_download.closeWindow();
			stat.texttype_print("%[color,yellow]Timed Out.%[color,default]\n");
			stat.closeWindow();
			outfile.delete();
			NSAutoreleasePool.pop(pool);
			return;
		}
		catch (SCCannotDownloadException e)
		{
			sendShioriEvent("OnURLDropFailure");
			prog_download.pbartype_setText1("["+e.getRespCode()+"]");
			prog_download.closeWindow();
			stat.texttype_print("The server returned %[color,yellow]"+e.getRespCode()+"%[color,default].\n");
			stat.closeWindow();
			outfile.delete();
			NSAutoreleasePool.pop(pool);
			return;
		}
		catch (Exception e)
		{
			sendShioriEvent("OnURLDropFailure");
			prog_download.pbartype_setText1("Error!");
			prog_download.closeWindow();
			stat.texttype_print("%[color,yellow]Error in downloading.%[color,default]\n");
			stat.closeWindow();
			outfile.delete();
			NSAutoreleasePool.pop(pool);
			return;
		}
		prog_download.pbartype_setText1("Done.");
		prog_download.closeWindow();
		
		stat.texttype_print("%[color,orange]downloading succeeded.%[color,default]");
		stat.closeWindow();
		
		sendShioriEvent("OnURLDropped",new String[] {outfile.getPath()});
		
		NSUserDefaults defaults = NSUserDefaults.standardUserDefaults();
		SCFoundation.sharedFoundation().install(outfile.getPath(),target,defaults.integerForKey("misc.delete_after_online_install") == 1);
		
		NSAutoreleasePool.pop(pool);
    }
    
    protected void sendShioriEvent(String eventName,String[] references)
    {
		if (target != null) target.doShioriEvent(eventName,references);
    }
    
    protected void sendShioriEvent(String eventName)
    {
		if (target != null) target.doShioriEvent(eventName);
    }
    
    public void waitUntilEnd()
    {
		while (isAlive()) {}
    }
}
