/*  $Id: SCSerikoLibrary.java,v 1.6 2004/01/30 08:04:25 phonohawk Exp $
	
    "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
    Copyright (C) 2001-2004 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
    SCSerikoLibrary
    SĂSERIKOV[PXLqt@C̒gǗ܂B

    o[W͎̒ʂłB
    0: SERIKO/1.0
    1: SERIKO/2.0
*/

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;
import java.io.*;
import java.util.*;
import java.util.regex.*;
import util.*;

public class SCSerikoLibrary {
    Vector seqGroups; // gSCSerikoSeqGroup
    
    public SCSerikoLibrary(SCShell shell) {
	seqGroups = new Vector();
		
	if (shell.getSurfaceDescriptions() != null) {
	    // surfaces.txt݂B
	    try {
		loadFromComprehensiveDefinitions(shell);
	    }
	    catch (Exception e) {
		System.err.println("Couldn't load surfaces.txt: {"+shell+"}");
		e.printStackTrace();
	    }
	}
	else {
	    // surfaces.txt݂ȂB
	    try {
		loadFromIndividualFile(shell);
	    }
	    catch (Exception e) {
		System.err.println("Couldn't load indivial surface definitions: {"+shell+"}");
		e.printStackTrace();
	    }
	}
    }

    protected void finalize() {
	Logger.log(this, Logger.DEBUG, "finalized");
    }
    
    public void dump() {
	final int n_groups = seqGroups.size();
	for (int i = 0;i < n_groups;i++) {
	    SCSerikoSeqGroup group = (SCSerikoSeqGroup)seqGroups.elementAt(i);
	    group.dump();
	}
    }
    
    protected void loadFromComprehensiveDefinitions(SCShell shell) {
	/*
	  SẴGgɂāÂ悤ȓs܂B
	  
	  GgGCAXf[^x[XtāAt炻̒ʂɓǂݍ݂܂B
	  f[^x[XɖGgKɏ]Ă邩ǂ𒲂ׂāA
	  ]Ăǂݍ݂܂B
	  łȂGg͖܂B
	*/
	SCBlockedDescription compDef = shell.getSurfaceDescriptions();

	// version
	int version = 0;
	if (compDef.isBlock("descript")) {
	    Pattern pat = Pattern.compile("^version\\s*,\\s*(\\d+)$");
	    Matcher matcher;
	    
	    Vector descript = (Vector)compDef.get("descript");
	    for (Iterator ite = descript.iterator(); ite.hasNext();) {
		String line = (String)ite.next();

		if ((matcher = pat.matcher(line)).matches()) {
		    try {
			version = Integer.parseInt(matcher.group(1));
		    }
		    catch (Exception e) {
			System.err.println("Warning: invalid version line ["+line+"]");
		    }
		    break;
		}
	    }
	}
	
	if (version != 0 && version != 1) {
	    System.err.println("Warning: unsupported SERIKO version ["+version+"]");
	    return;
	}
	
	for (Enumeration keys = compDef.keys();keys.hasMoreElements();) {
	    String key = (String)keys.nextElement();
	    
	    String inv_result = shell.getAliasManager().inverseSearch(key);
	    if (inv_result != null) { // 
		// L[p[XB
		// surface[id]ƂȂĂ锤B
		int id = -1;
		try {
		    id = Integer.parseInt( inv_result.substring(7) );
		}
		catch (Exception e) {} // G[N-1̂܂܁B
		
		if (id != -1) { // FB
		    Object element = compDef.get(key);
		    if (element instanceof Vector) {
			seqGroups.add(new SCSerikoSeqGroup(version, (Vector)element, id));
		    }
		}
	    }
	    else { // tĂȂ
		// Kɏ]Ă邩H
		if (key.startsWith("surface")) {
		    int id = -1;
		    try {
			id = Integer.parseInt( key.substring(7) );
		    }catch (Exception e) {}
		    
		    if (id != -1) { // FB
			Object element = compDef.get(key);
			if (element instanceof Vector) {
			    seqGroups.add(new SCSerikoSeqGroup(version, (Vector)element, id));
			}
		    }
		}
	    }
	}
    }
    
    protected void loadFromIndividualFile(SCShell shell) {
	/*
	  SẴt@CɂāÂ悤ȓs܂B
	  
	  t@CGCAXf[^x[XtāA炻̒ʂɓǂݍ݂܂B
	  f[^x[Xɖt@CSERIKOV[PXf[^̖Kɏ]Ă邩ǂ𒲂ׂāA
	  ]Ăǂݍ݂܂B
	  łȂt@C͖܂B
	*/
	File[] files = shell.getRootDir().listFiles();
	for (int i = 0;i < files.length;i++) {
	    String fname = files[i].getName();
	    String lowercase_fname = files[i].getName().toLowerCase(); // p[XpɏɕςĂ܂̂ŁApXɎgĂ͂܂B
	    if (!lowercase_fname.endsWith("a.txt") && !lowercase_fname.endsWith("a.dat")) continue;
	    
	    String familyname; // t@Ca.txta.dat
	    if (lowercase_fname.lastIndexOf("a.txt") != -1) {
		familyname = fname.substring(0, lowercase_fname.lastIndexOf("a.txt"));
	    }
	    else {
		familyname = fname.substring(0, lowercase_fname.lastIndexOf("a.dat"));
	    }
	    
	    String inv_result = shell.getAliasManager().inverseSearch(familyname);
	    if (inv_result != null) { // 
		// L[p[XB
		// surface[id]ƂȂĂ锤B
		int id = -1;
		try {
		    id = Integer.parseInt( inv_result.substring(7) );
		}
		catch (Exception e) {} // G[N-1̂܂܁B
		
		if (id != -1) { // FB
		    seqGroups.add(new SCSerikoSeqGroup(files[i],id));
		}
	    }
	    else { // tĂȂ
		// Kɏ]Ă邩H
		if (lowercase_fname.startsWith("surface")) { // a.txtŏI邱Ƃ͕ĂB
		    int id = -1;
		    try {
			id = Integer.parseInt( familyname.substring(7) );
		    } catch (Exception e) {}
		    
		    if (id != -1) { // FB
			seqGroups.add(new SCSerikoSeqGroup(files[i],id));
		    }
		}
	    }
	}
    }
    
    public SCSerikoSeqGroup findSeqGroup(int id) {
	// w肳ꂽV[PXO[vIDŁAV[PXO[v܂B
	// ȂnullԂ܂B
	int n_groups = seqGroups.size();
	for (int i = 0;i < n_groups;i++) {
	    SCSerikoSeqGroup seqgroup = (SCSerikoSeqGroup)seqGroups.elementAt(i);
	    if (seqgroup.groupId() == id) {
		return seqgroup;
	    }
	}
	
	return null;
    }
}
