/*  $Id: SCShell.java,v 1.7 2004/01/20 17:34:16 phonohawk Exp $
	
    "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
    Copyright (C) 2001 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;
import java.io.*;
import util.*;

public class SCShell {
    SCSession session;
    
    SCDescription descManager;
    SCAliasManager aliasFileNameTable;
    SCAliasNameTable aliasNameTable;
    SCSurfaceServer surfserver;
    SCSeriko seriko;
    SCBlockedDescription surfaceDescriptions; // surfaces.txt݂ȂnullɂȂĂB
    
    String dirname;
    String shellname;
    File shellRootDir;
    File readmeFile;
    
    String selfname;
    String keroname;
    
    public SCShell(SCSession session,String dirname) {
	this(session,dirname,null);
    }
    
    public SCShell(SCSession session,String dirname,SCStatusWindowController status) {
	this.session = session;
	this.dirname = dirname;
	
	shellRootDir = new File(session.getGhostDir(),"shell/"+dirname);
	descManager = new SCDescription(new File(shellRootDir,"descript.txt"));
	//descManager.setTag(session.toString());
	shellname = descManager.getStrValue("name");
	printstat(status,"The shell of the ghost,\n");
	printstat(status,"shell name: %[color,yellow]"+shellname+"%[color,default]\n");
	
	File surfaces_txt = new File(shellRootDir,"surfaces.txt");
	if (surfaces_txt.exists()) {
	    surfaceDescriptions = new SCBlockedDescription(surfaces_txt);
	} else {
	    surfaceDescriptions = null;
	}
	
	selfname = descManager.getStrValue("sakura.name");
	keroname = descManager.getStrValue("kero.name");
	File aliasTxt = new File(shellRootDir,"alias.txt");
	aliasFileNameTable = new SCAliasManager(aliasTxt);
	aliasNameTable = new SCAliasNameTable();
	aliasNameTable.load(aliasTxt); // alias.txtǂ܂Ă
	aliasNameTable.load(surfaces_txt); // surfaces.txtǂ܂B
	readmeFile = new File(shellRootDir,descManager.exists("readme") ? descManager.getStrValue("readme") : "readme.txt");
	
	// T[tBXT[o[N
	printstat(status,"%[color,orange]loading shell... ");
	long start_time = System.currentTimeMillis();
	surfserver = new SCSurfaceServer(this,shellRootDir);
	printstat(status,"%[color,white]done.("+(System.currentTimeMillis()-start_time)+"ms)%[color,default]\n");
	if (!surfserver.hasLoadAllSurfaces()) {
	    printstat(status,"Actual surfaces will be loaded when required.\n");
	}
    }
    protected void printstat(SCStatusWindowController status,String msg) {
	if (status != null) status.texttype_print(msg);
    }
    
    public void start() {
	// phase9.10ȍ~ANɃVF\ȂB

	// SERIKON
	seriko = new SCSeriko(session,this,shellRootDir);
    }
    
    public void close() {
	session = null;
	descManager = null;
	aliasFileNameTable = null;
	aliasNameTable = null;
	surfserver = null;
	seriko.terminate(); seriko = null;
	surfaceDescriptions = null;
    }
    
    public SCDescription getDescManager() {
        return descManager;
    }
    
    public SCAliasManager getAliasManager() {
        return aliasFileNameTable;
    }
    
    public SCAliasNameTable getAliasNameTable() {
	return aliasNameTable;
    }
    
    public SCBlockedDescription getSurfaceDescriptions() {
	return surfaceDescriptions;
    }
    
    public SCSurfaceServer getSurfaceServer() {
        return surfserver;
    }
    
    public SCSeriko getSeriko() {
	return seriko;
    }
    
    public String getDirName() {
	return dirname;
    }
    
    public File getRootDir() {
	return shellRootDir;
    }
    
    public File getReadmeFile() {
	return readmeFile;
    }
    
    public String getShellName() {
	String nameFromDesc = descManager.getStrValue("name");
	return (nameFromDesc == null ? dirname : nameFromDesc);
    }
    
    public String getSelfName() {
	// w肳ĂȂnullԂ܂B
	return selfname;
    }
    
    public String getKeroName() {
	// w肳ĂȂnullԂ܂B
	return keroname;
    }

    String id;
    public String getIdentification() {
	// sakura, keronullłnullԂB
	if (selfname == null && keroname == null) {
	    return null;
	}
	
	if (id == null) {
	    StringBuffer buf = new StringBuffer(30);
	    if (selfname != null) {
		buf.append(selfname);
	    }
	    buf.append(',');
	    if (keroname != null) {
		buf.append(keroname);
	    }
	    id = buf.toString();
	}
	return id;
    }

    public String toString() {
	StringBuffer buf = new StringBuffer();
	buf.append("session: {").append(session).append('}');
	buf.append("; dirname: ").append(dirname);
	buf.append("; seriko: {").append(seriko).append('}');
	return buf.toString();
    }

    protected void finalize() {
	//Logger.log(this, Logger.DEBUG, "finalized");
    }
}
