/*
 "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
 Copyright (C) 2001-2003 phonohawk

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package aya.eval;
import aya.eval.*;
import aya.*;
import java.util.*;
import java.util.regex.*;
import java.io.*;
import java.lang.reflect.*;

// 大域的名前空間は文インスタンス一つにつき一つだけ存在する。

public class GlobalNamespace extends Namespace
{
	static Hashtable method_cache; // {システム変数名 => Method}
	static {
		method_cache = new Hashtable();
	}
	
	Hashtable sysvar_cache = null; // {システム変数名 => Variable}
	
	public void saveToFile(File f) {
		try {
			saveToStream(new FileOutputStream(f));
		}
		catch (Exception e) {
			System.err.println("Couldn't save global namespace.");
			e.printStackTrace();
		}
	}
	
	static final Pattern pat_doublequote = Pattern.compile("\"");
	public void saveToStream(OutputStream os) {
		try {
			BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(os,Aya.CHARSET));
			
			// 関数 @@@SAVEDATA@@@ を定義し、これを評価する事で復帰出来るようにする。
			bw.write("@@@SAVEDATA@@@ {\r\n");
			for (Enumeration keys = hash.keys(); keys.hasMoreElements(); ) {
				String varname = (String)keys.nextElement();
				Variable v = (Variable)hash.get(varname);
				
				Value val = v.getValue();
				if (val.isString()) {
					// 文字列中のダブルクォートは、%ASC(34)に置換する。
					String content = pat_doublequote.matcher(val.getString()).replaceAll("%ASC(34)");
					bw.write("    "+varname+" = \""+content+"\";\r\n");
					
					// 配列として見た時に、要素が二つ以上あれば、
					// SETSEPARATOR(var,delim);を記録する。
					// デリミタがカンマのままであっても省略しない。
					// 将来デリミタのデフォルトが変わるかも知れないからである。
					if (v.getArraySize() > 1) {
						// デリミタ中のダブルクォートは、%ASC(34)に置換する。
						String delim = pat_doublequote.matcher(v.getSeparator()).replaceAll("%ASC(34)");
						bw.write("    SETSEPARATOR("+varname+",\""+delim+"\");\r\n");
					}
				}
				else {
					bw.write("    "+varname+" := "+val.getReal()+";\r\n");
				}
			}
			bw.write("}\r\n");
			bw.flush();
		}
		catch (Exception e) {
			System.err.println("Couldn't save global namespace.");
			e.printStackTrace();
		}
	}
	
	public Variable get(String name) {
		String method_name = "sysvar_"+name;
	
		// メソッドがキャッシュされているか？
		Method m = (Method)method_cache.get(method_name);
		if (m == null) {
			// されていない。メソッドが存在するか？
			try {
				m = this.getClass().getMethod(method_name,null);
				method_cache.put(method_name,m);
			}
			catch (Exception e) {}
		}
		
		// メソッドがあるなら呼ぶ。
		if (m != null) {
			try {
				return (Variable)m.invoke(this,null);
			}
			catch (Exception e) {
				System.err.println("Couldn't get value of system variable "+name+".");
				e.printStackTrace();
				return null;
			}
		}
		
		// 無ければ通常変数。
		return super.get(name);
	}
	
	public Variable define(String name) {
		Variable v = get(name);
		if (v == null) {
			return super.define(name);
		}
		else {
			return v;
		}
	}
	
	protected Variable defineSysVar(String name) {
		// sysvar_cacheがnullのままなら、Hashtableを作る。
		if (sysvar_cache == null) {
			sysvar_cache = new Hashtable();
		}
		
		Variable v = (Variable)sysvar_cache.get(name);
		if (v == null) {
			// まだ無いので、作る。
			v = new Variable(new Value(""));
			sysvar_cache.put(name,v);
		}
		return v;
	}
	
	// ---------- これらの関数は間接的に呼ばれる ----------
	public Variable sysvar_year() {
		Calendar cal = Calendar.getInstance();
		
		Variable v = defineSysVar("year");
		v.unlock();
		v.setValue(new Value(cal.get(Calendar.YEAR)));
		v.lock();
		return v;
	}
	
	public Variable sysvar_month() {
		Calendar cal = Calendar.getInstance();
		
		Variable v = defineSysVar("month");
		v.unlock();
		v.setValue(new Value(cal.get(Calendar.MONTH)+1));
		v.lock();
		return v;
	}
	
	public Variable sysvar_day() {
		Calendar cal = Calendar.getInstance();
		
		Variable v = defineSysVar("day");
		v.unlock();
		v.setValue(new Value(cal.get(Calendar.DAY_OF_MONTH)));
		v.lock();
		return v;
	}
	
	public Variable sysvar_weekday() {
		Calendar cal = Calendar.getInstance();
		cal.setFirstDayOfWeek(Calendar.SUNDAY);
		
		Variable v = defineSysVar("weekday");
		v.unlock();
		v.setValue(new Value(cal.get(Calendar.DAY_OF_WEEK)));
		v.lock();
		return v;
	}
	
	public Variable sysvar_hour() {
		Calendar cal = Calendar.getInstance();
		
		Variable v = defineSysVar("hour");
		v.unlock();
		v.setValue(new Value(cal.get(Calendar.HOUR_OF_DAY)));
		v.lock();
		return v;
	}
	
	public Variable sysvar_12hour() {
		Calendar cal = Calendar.getInstance();
		
		Variable v = defineSysVar("12hour");
		v.unlock();
		v.setValue(new Value(cal.get(Calendar.HOUR_OF_DAY) % 12));
		v.lock();
		return v;
	}
	
	public Variable sysvar_ampm() {
		Calendar cal = Calendar.getInstance();
		
		Variable v = defineSysVar("ampm");
		v.unlock();
		v.setValue(new Value(cal.get(Calendar.HOUR_OF_DAY) >= 12 ? 1 : 0)); // 0が午前、1が午後。
		v.lock();
		return v;
	}
	
	public Variable sysvar_minute() {
		Calendar cal = Calendar.getInstance();
		
		Variable v = defineSysVar("minute");
		v.unlock();
		v.setValue(new Value(cal.get(Calendar.MINUTE)));
		v.lock();
		return v;
	}
	
	public Variable sysvar_second() {
		Calendar cal = Calendar.getInstance();
		
		Variable v = defineSysVar("second");
		v.unlock();
		v.setValue(new Value(cal.get(Calendar.SECOND)));
		v.lock();
		return v;
	}
	
	// これらは取得する方法を知らない。
	// uptimeのmanには/machを読めば分かるような事が書いてあるが、どうやって読むんだ？
	public Variable sysvar_systemuptickcount() {
		return unimplemented("systemuptickcount");
	}
	public Variable sysvar_systemuptime() {
		return unimplemented("systemuptime");
	}
	public Variable sysvar_systemuphour() {
		return unimplemented("systemuphour");
	}
	public Variable sysvar_systemupminute() {
		return unimplemented("systemupminute");
	}
	public Variable sysvar_systemupsecond() {
		return unimplemented("systemupsecond");
	}
	
	// vm_statの出力をパースすれば分かるだろうが、面倒。
	public Variable sysvar_memoryload() {
		return unimplemented("memoryload");
	}
	public Variable sysvar_memorytotalphys() {
		return unimplemented("memorytotalphys");
	}
	public Variable sysvar_memorytotalvirtual() {
		return unimplemented("memorytotalvirtual");
	}
	public Variable sysvar_memoryavailvirtual() {
		return unimplemented("memoryavailvirtual");
	}
	
	protected Variable unimplemented(String name) {
		Variable v = defineSysVar(name);
		v.unlock();
		v.setValue(new Value(0));
		v.lock();
		return v;
	}
}
