/*
 "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
 Copyright (C) 2001-2003 phonohawk

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package aya.eval;
import aya.eval.*;
import aya.*;
import java.util.*;

public class Variable
{
	String separator;
	boolean read_only;
	
	// ↓この二つの値は、互いのキャッシュである。
	Vector array; // Value又はVariablePointerの配列。
	Value value; // 変数の値。
	// ↑どちらか一方はnullで良い。
	
	public Variable(Vector array)
	{
		// array: String又はVariablePointerの配列。
		this.separator = ",";
		this.read_only = false;
		this.array = array;
		this.value = null;
	}
	
	public Variable(Value value)
	{
		this.separator = ",";
		this.read_only = false;
		this.array = null;
		this.value = value;
	}
	
	public Variable lock() {
		read_only = true;
		return this;
	}
	
	public Variable unlock() {
		read_only = false;
		return this;
	}
	
	public Value getValue() {
		if (value == null) {
			// フラットな値がキャッシュされていない。
			StringBuffer flat = new StringBuffer();
			for (int i = 0; i < array.size(); i++) {
				if (i > 0) {
					flat.append(separator);
				}
				
				Object elem = array.elementAt(i);
				if (elem instanceof Value) {
					flat.append(((Value)elem).getString());
				}
				else if (elem instanceof VariablePointer) {
					flat.append(((VariablePointer)elem).fetch());
				}
				else {
					throw new RuntimeException("Internal Error: an object of "+elem.getClass().getName()+" was in array part of Variable illegally.");
				}
			}
			value = new Value(flat.toString());
		}
		return value;
	}
	
	public Value getValue(int index) {
		if (array == null) {
			// 配列がキャッシュされていない。
			array = new Vector();
			
			String src = value.getString();
			for (int start = 0;;) {
				int pos = src.indexOf(separator,start);
				if (pos == -1) {
					// もう無い。
					array.add(new Value(src.substring(start)));
					break;
				}
				
				array.add(new Value(src.substring(start,pos)));
				start = pos + separator.length();
			}
		}
		
		// 配列の要素数をindexが越えていたら、常に空文字列を返す。
		if (index >= array.size()) {
			return new Value("");
		}
		else {
			Object obj = array.elementAt(index);
			if (obj instanceof Value) {
				return (Value)obj;
			}
			else if (obj instanceof VariablePointer) {
				return ((VariablePointer)obj).fetch();
			}
			else {
				throw new RuntimeException("Internal Error: an object of "+obj.getClass().getName()+" was in array part of Variable illegally.");
			}
		}
	}
	
	public Variable setValue(Value value) {
		if (read_only) {
			throw new RuntimeException("This variable is read-only. Do not modify this.\n");
		}
	
		this.array = null;
		this.value = value;
		return this;
	}
	
	public Variable setValue(int index,Value value) throws Exception {
		if (read_only) {
			throw new RuntimeException("This variable is read-only. Do not modify this.\n");
		}
		
		this.value = null;
		if (array == null) {
			getValue(0); // 配列を作ってキャッシュ
		}
		
		// indexは実際の配列のサイズを越えているか？
		if (index >= array.size()) {
			// この要素が入る所まで空文字列を追加し続ける。
			for (int i = array.size(); i < index; i++) {
				array.add("");
			}
			// この要素を入れる。
			array.add(value.getString());
		}
		else {
			// 元の値がValueか暗黙のVariablePointerなら、単純に置き換える。
			// 明示的なVariablePointerなら、参照先を書き換える。
			Object original = array.elementAt(index);
			if (original instanceof Value) {
				array.set(index,value);
			}
			else if (original instanceof VariablePointer) {
				VariablePointer vptr = (VariablePointer)original;
				if (vptr.isTacit()) {
					array.set(index,value);
				}
				else {
					vptr.store(value);
				}
			}
			else {
				throw new Exception("Internal Error: an object of "+original.getClass().getName()+" was in array part of Variable illegally.");
			}
		}
		return this;
	}
	
	public String getSeparator() {
		return separator;
	}
	
	public Variable setSeparator(String sep) {
		if (read_only) {
			throw new RuntimeException("This variable is read-only. Do not modify this.\n");
		}
		
		separator = sep;
		array = null; // 配列のキャッシュをクリア
		return this;
	}
	
	public long getArraySize() {
		if (array == null) {
			getValue(0); // 配列を作ってキャッシュ
		}
		return array.size();
	}
}
