/*
 "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
 Copyright (C) 2001-2003 phonohawk

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package aya.eval;
import aya.eval.*;
import aya.node.*;
import aya.*;
import java.util.*;

public class VariablePointer
{
	Namespace ns;
	VariableReference vref;
	boolean index_exists;
	int evaluated_index;
	boolean tacit;
	
	public VariablePointer(Namespace ns,VariableReference vref) throws Exception {
		this(ns,vref,false);
	}
	
	public VariablePointer(Namespace ns,VariableReference vref,boolean tacit) throws Exception {
		// tacit: この変数ポインタが、単一の変数参照により暗黙に作られたものであれば、trueを渡す。
		this.ns = ns;
		this.vref = vref;
		this.tacit = tacit;
		
		if (vref.isGlobal() && !(ns instanceof GlobalNamespace)) {
			throw new Exception("Internal Error: created VariablePointer points global variable, but its namespace is not global. The vref is "+vref.toString());
		}
		
		// ポインタが作られた時点で、添字は評価してしまう。
		// 後になってから添字の値が変化するようでは困るので。
		if (vref.getIndex() != null) {
			index_exists = true;
			Value idx = vref.getIndex().eval(ns);
			if (idx.isInteger()) {
				evaluated_index = (int)idx.getInteger();
			}
			else {
				throw new Exception("Index of pseudo-array must be numeric value.");
			}
		}
		else {
			index_exists = false;
			evaluated_index = -1;
		}
	}
	
	public boolean isTacit() {
		return tacit;
	}
	
	public VariableReference getVariableReference() {
		return vref;
	}
	
	public Namespace getNamespace() {
		return ns;
	}
	
	public Variable getVariable() {
		return ns.define(vref.getName());
	}
	
	public VariablePointer store(Value val) throws Exception {
		// このポインタが指す先に代入する。
		Variable v = ns.define(vref.getName());
		
		if (index_exists) {
			// 簡易配列の添字がある。
			v.setValue(evaluated_index,val);
		}
		else {
			v.setValue(val);
		}
		return this;
	}
	
	public Value fetch() {
		Variable v = ns.define(vref.getName());
		
		if (index_exists) {
			// 簡易配列の添字がある。
			return v.getValue(evaluated_index);
		}
		else {
			return v.getValue();
		}
	}
}
