/*
 "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
 Copyright (C) 2001-2003 phonohawk

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package aya.node;
import aya.*;
import aya.node.*;
import aya.eval.*;
import java.util.*;

public class Block
{
	public static final int OPT_NONE = 0;
	public static final int OPT_NONOVERLAP = 1;
	public static final int OPT_SEQUENTIAL = 2;
	
	Aya aya;
	int option;
	Vector statements; // [Statement,Statement,...]
	
	String last_group_pattern; // Ōɕ]̃p^[
	
	HashSet nonoverlap_table; // {oH}
	// oH: 2ԖځA4ԖځA0Ԗڂ̏ɑI񂾂ƂAoH"2-4-0"
	// 		 ̌oHԍ#ŕ\B
	
	LinkedList sequential_list; // [oH]
	
	public Block(Aya aya)
	{
		this.aya = aya;
		this.option = 0;
		this.statements = new Vector();
		this.last_group_pattern = null;
		this.nonoverlap_table = null;
		this.sequential_list = null;
	}
	
	public Block setOption(String o) {
		if (o != null) {
			if (o.equals("nonoverlap")) {
				option = OPT_NONOVERLAP;
				
				last_group_pattern = "";
				nonoverlap_table = new HashSet();
				sequential_list = null;
			}
			else if (o.equals("sequential")) {
				option = OPT_SEQUENTIAL;
				
				last_group_pattern = "";
				nonoverlap_table = null;
				sequential_list = new LinkedList();
			}
			else {
				try {
					throw new Exception("Option of block \""+o+"\" is not supported.");
				}
				catch (Exception e) {}
			}
		}
		return this;
	}
	
	public int getOption() {
		return option;
	}
	
	public Vector getStatements() {
		return statements;
	}
	
	public Block add(Statement s) {
		statements.add(s);
		return this;
	}
	
	/*
		ubN]ɂ͎l̕@܂B
		Ȃ킿Aʏ]Anonoverlap]Asequential]Aswitch]łB
	*/
	
	public Value eval(Namespace ns) throws Exception {
		// evalnullԂꍇB
		return eval(ns,-1);
	}
	public Value eval(Namespace ns,int switch_index) throws Exception {
		Vector resultgroups = new Vector(); // [Vector,...]
		
		Vector currentgroup = new Vector(); // [Value,...]
		resultgroups.add(currentgroup);
		
		for (int i = 0; i < statements.size(); i++) {
			Statement statement = (Statement)statements.elementAt(i);
			
			if (statement.isSeparator()) {
				// o͊mqB
				// ̃O[vIB̃O[vցB
				currentgroup = new Vector();
				resultgroups.add(currentgroup);
			}
			else if (statement.isBreak()) {
				throw statement.newBreakOccurrence();
			}
			else if (statement.isContinue()) {
				throw statement.newContinueOccurrence();
			}
			else if (statement.isReturn()) {
				// ŏIB
				return constructResult(resultgroups,switch_index);
			}
			else {
				Object result = statement.eval(ns);
				if (result != null) {
					if (result instanceof Value) {
						currentgroup.add(result);
					}
					else if (result instanceof Vector) {
						currentgroup.addAll((Vector)result);
					}
					else {
						throw new Exception("Internal Error: Statement#eval() returned object of "+result.getClass().getName()+" illegally.");
					}
				}
			}
		}
		
		return constructResult(resultgroups,switch_index);
	}
	protected Value constructResult(Vector resultgroups,int switch_index) throws Exception {
		if (option == OPT_NONE) {
			Value result = null;
			
			for (int i = 0; i < resultgroups.size(); i++) {
				Vector group = (Vector)resultgroups.elementAt(i);
				Value val_of_group = null;
				
				if (switch_index >= 0) {
					if (switch_index < group.size()) {
						val_of_group = (Value)group.elementAt(switch_index);
					}
					else {
						val_of_group = new Value("");
					}
				}
				else {
					if (group.size() > 0) {
						int idx = (int)(Math.random() * Integer.MAX_VALUE) % group.size();
						val_of_group = (Value)group.elementAt(idx);
					}
				}
				
				if (val_of_group != null) {
					if (result == null) {
						result = val_of_group;
					}
					else {
						result.setString(result.getString() + val_of_group.getString());
					}
				}
			}
			
			return result;
		}
		else {
			if (option == OPT_NONOVERLAP) {
				// O̕]̃p^[ƍ̃p^[ςĂ邩H
				String pat = generateGroupPattern(resultgroups);
				if (!last_group_pattern.equals(pat)) {
					// ςĂ̂ŁAde[u蒼B
					reconstructNonoverlapTable(resultgroups);
					last_group_pattern = pat;
				}
				
				// resultgroupsłȂɂւ炸nonoverlap_tableɂȂĂA
				// ͂de[u蒼B
				if (resultgroups.size() > 0 && nonoverlap_table.size() == 0) {
					reconstructNonoverlapTable(resultgroups);
				}
				
				// ܂gĂȂoHIԁB
				Object[] paths = nonoverlap_table.toArray();
				int nth = (int)(Math.random() * Integer.MAX_VALUE) % paths.length;
				
				// I񂾌oH͍폜B
				nonoverlap_table.remove(paths[nth]);
				
				return selectWithPath(resultgroups,(String)paths[nth]);
			}
			else if (option == OPT_SEQUENTIAL) {
				// O̕]̃p^[ƍ̃p^[ςĂ邩H
				String pat = generateGroupPattern(resultgroups);
				if (!last_group_pattern.equals(pat)) {
					// ςĂ̂ŁAXg蒼B
					reconstructSequentialList(resultgroups);
					last_group_pattern = pat;
				}
				
				// resultgroupsłȂɂւ炸sequential_listɂȂĂA
				// ͂菇Xg蒼B
				if (resultgroups.size() > 0 && sequential_list.size() == 0) {
					reconstructSequentialList(resultgroups);
				}
				
				// 擪̌oHIŁA폜B
				String path = (String)sequential_list.removeFirst();
				
				return selectWithPath(resultgroups,path);
				
			}
			else {
				throw new Exception("Block option '"+option+"' is not supported.");
			}
		}
	}
	protected static String generateGroupPattern(Vector resultgroups) {
		/*
			ႦΎ̂悤ȌʂɂȂꍇÅ֐
			"2/10/3"ƂԂ܂B
			
			{
				"foo"
				"bar"
				--
				1
				2
				..
				10
				--
				"aaa"
				"bbb"
				"ccc"
			}
		*/
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < resultgroups.size(); i++) {
			Vector group = (Vector)resultgroups.get(i);
			
			if (i != 0) {
				buf.append('/');
			}
			
			buf.append(Integer.toString(group.size()));
		}
		return buf.toString();
	}
	
	protected void reconstructNonoverlapTable(Vector resultgroups) {
		nonoverlap_table.clear();
		
		// \ȑSĂ̏ꍇ̌oHAHashSetɓB
		if (resultgroups.size() > 0) {
			Vector paths = makeNonoverlapPath(resultgroups,0);
			for (int i = 0; i < paths.size(); i++) {
				nonoverlap_table.add(paths.get(i));
			}
		}
	}
	protected void reconstructSequentialList(Vector resultgroups) {
		sequential_list.clear();
		
		// \ȑSĂ̏ꍇ̌oHALinkedListɌqB
		if (resultgroups.size() > 0) {
			Vector paths = makeNonoverlapPath(resultgroups,0);
			for (int i = 0; i < paths.size(); i++) {
				sequential_list.addLast(paths.get(i));
			}
		}
	}
	protected Vector makeNonoverlapPath(Vector resultgroups,int level) {
		Vector result = new Vector();
		Vector current = (Vector)resultgroups.get(level);
		
		// 艺̃x邩H
		if (resultgroups.size() > level + 1) {
			// ̂ŁAċAIɌoHB
			Vector lower = makeNonoverlapPath(resultgroups,level + 1);
			
			// ꂽSĂ̌oHɑ΂āÃO[v̗vfɑΉoHVɍB
			if (current.size() > 0) {
				for (int i = 0; i < lower.size(); i++) {
					for (int j = 0; j < current.size(); j++) {
						result.add(j+"-"+(String)lower.get(i));
					}
				}
			}
			else {
				for (int i = 0; i < lower.size(); i++) {
					result.add("#-"+(String)lower.get(i));
				}
			}
		}
		else {
			// ̂ŁÃO[v̗vfoHƂB
			if (current.size() > 0) {
				for (int i = 0; i < current.size(); i++) {
					result.add(Integer.toString(i));
				}
			}
			else {
				result.add("#");
			}
		}
		
		return result;
	}
	protected Value selectWithPath(Vector resultgroups,String path) {
		// pathp[X
		Value result = null;
		
		StringTokenizer st = new StringTokenizer(path,"-");
		for (int i = 0; st.hasMoreTokens(); i++) {
			Vector current = (Vector)resultgroups.get(i);
			String index = st.nextToken();
			
			if (current.equals("#")) {
				// ̃ubN͋BցB
			}
			else {
				try {
					Value val = (Value)current.get(Integer.parseInt(index));
					if (result == null) {
						result = val;
					}
					else {
						result.setString(result.getString() + val.getString());
					}
				} catch (Exception e) {}
			}
		}
		
		return result;
	}
}
