/*  $Id: ZenHanTranslator.java,v 1.3 2004/01/29 12:14:45 phonohawk Exp $
    
    "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
    Copyright (C) 2001-2004 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package satori;
import satori.*;

public class ZenHanTranslator {
    public static final char ZENKAKU_AST = '\uff0a'; // 
    public static final char ZENKAKU_COLON = '\uff1a'; // F
    public static final char ZENKAKU_AT = '\uff20'; // 
    public static final char ZENKAKU_SHARP = '\uff03'; // 
    public static final char ZENKAKU_LT = '\uff1c'; // 
    public static final char ZENKAKU_GT = '\uff1e'; // 
    public static final char ZENKAKU_PARENTHESIS_OPEN = '\uff08'; // i
    public static final char ZENKAKU_PARENTHESIS_CLOSE = '\uff09'; // j
    public static final char ZENKAKU_PERCENT = '\uff05'; // 
    public static final char ZENKAKU_DOLLAR = '\uff04'; // 
    public static final char ZENKAKU_R = '\uff32'; // q
    public static final char ZENKAKU_H = '\uff28'; // g
    public static final char ZENKAKU_S = '\uff33'; // r
    public static final char ZENKAKU_UNDERSCORE = '\uff3f'; // Q
    public static final char ZENKAKU_MINUS = '\uff0d'; // ]
    public static final char ZENKAKU_EXCLAM = '\uff01'; // I
    public static final char ZENKAKU_CARET = '\uff3e'; // O
    public static final char ZENKAKU_SLASH = '\uff0f'; // ^
    public static final char ZENKAKU_PLUS = '\uff0b'; // {
    public static final char ZENKAKU_EQUAL = '\uff1d'; // 
    public static final char ZENKAKU_AND = '\uff06'; // 
    public static final char ZENKAKU_VERTBAR = '\uff5c'; // b
    public static final char ZENKAKU_NAMI = '\u301c'; // `
    public static final char ZENKAKU_YEN = '\uffe5'; // 
    public static final char ZENKAKU_CLOSS = '\u00d7'; // ~
    public static final char ZENKAKU_DIV = '\u00f7'; // 
    public static final char[] ZENKAKU_NUM = {
	'\uff10','\uff11','\uff12','\uff13','\uff14',
	'\uff15','\uff16','\uff17','\uff18','\uff19'}; // O`X
    
    public static String convZenToHan(String src) {
	// SpXy[X͕ϊ܂B
	final int len = src.length();
	StringBuffer buf = new StringBuffer(src);
	for (int i = 0;i < len;i++) {
	    char c = buf.charAt(i);
	    switch (c) {
	    case '\uff10': case '\uff11': case '\uff12': case '\uff13': case '\uff14': 
	    case '\uff15': case '\uff16': case '\uff17': case '\uff18': case '\uff19': 
		buf.setCharAt(i,(char)(c - '\uff10' + '0'));
		break;
	    case ZENKAKU_AST: buf.setCharAt(i,'*'); break;
	    case ZENKAKU_COLON: buf.setCharAt(i,':'); break;
	    case ZENKAKU_AT: buf.setCharAt(i,'@'); break;
	    case ZENKAKU_SHARP: buf.setCharAt(i,'#'); break;
	    case ZENKAKU_LT: buf.setCharAt(i,'<'); break;
	    case ZENKAKU_GT: buf.setCharAt(i,'>'); break;
		//case ZENKAKU_PARENTHESIS_OPEN: buf.setCharAt(i,'('); break;
		//case ZENKAKU_PARENTHESIS_CLOSE: buf.setCharAt(i,')'); break;
	    case ZENKAKU_PERCENT: buf.setCharAt(i,'%'); break;
	    case ZENKAKU_DOLLAR: buf.setCharAt(i,'$'); break;
	    case ZENKAKU_R: buf.setCharAt(i,'R'); break;
	    case ZENKAKU_H: buf.setCharAt(i,'H'); break;
	    case ZENKAKU_S: buf.setCharAt(i,'S'); break;
	    case ZENKAKU_UNDERSCORE: buf.setCharAt(i,'_'); break;
	    case ZENKAKU_MINUS: buf.setCharAt(i,'-'); break;
	    case ZENKAKU_EXCLAM: buf.setCharAt(i,'!'); break;
	    case ZENKAKU_CARET: buf.setCharAt(i,'^'); break;
	    case ZENKAKU_SLASH: buf.setCharAt(i,'/'); break;
	    case ZENKAKU_PLUS: buf.setCharAt(i,'+'); break;
	    case ZENKAKU_EQUAL: buf.setCharAt(i,'='); break;
	    case ZENKAKU_AND: buf.setCharAt(i,'&'); break;
	    case ZENKAKU_VERTBAR: buf.setCharAt(i,'|'); break;
	    case ZENKAKU_NAMI: buf.setCharAt(i,'~'); break;
	    case ZENKAKU_YEN: buf.setCharAt(i,'\u00a5'); break;
	    case ZENKAKU_CLOSS: buf.setCharAt(i,'*'); break;
	    case ZENKAKU_DIV: buf.setCharAt(i,'/'); break;
	    }
	}
	
	return buf.toString();
    }
	
    public static String convHanToZen(String src) {
	// pXy[X͕ϊ܂B
	final int len = src.length();
	StringBuffer buf = new StringBuffer(src);
	for (int i = 0;i < len;i++) {
	    char c = buf.charAt(i);
	    switch (c) {
	    case '0': case '1': case '2': case '3': case '4':
	    case '5': case '6': case '7': case '8': case '9':
		buf.setCharAt(i,(char)(c - '0' + '\uff10'));
		break;
	    case '*': buf.setCharAt(i,ZENKAKU_AST); break;
	    case ':': buf.setCharAt(i,ZENKAKU_COLON); break;
	    case '@': buf.setCharAt(i,ZENKAKU_AT); break;
	    case '#': buf.setCharAt(i,ZENKAKU_SHARP); break;
	    case '<': buf.setCharAt(i,ZENKAKU_LT); break;
	    case '>': buf.setCharAt(i,ZENKAKU_GT); break;
		//case '(': buf.setCharAt(i,ZENKAKU_PARENTHESIS_OPEN); break;
		//case ')': buf.setCharAt(i,ZENKAKU_PARENTHESIS_CLOSE); break;
	    case '%': buf.setCharAt(i,ZENKAKU_PERCENT); break;
	    case '$': buf.setCharAt(i,ZENKAKU_DOLLAR); break;
	    case 'R': buf.setCharAt(i,ZENKAKU_R); break;
	    case 'H': buf.setCharAt(i,ZENKAKU_H); break;
	    case 'S': buf.setCharAt(i,ZENKAKU_S); break;
	    case '_': buf.setCharAt(i,ZENKAKU_UNDERSCORE); break;
	    case '-': buf.setCharAt(i,ZENKAKU_MINUS); break;
	    case '!': buf.setCharAt(i,ZENKAKU_EXCLAM); break;
	    case '^': buf.setCharAt(i,ZENKAKU_CARET); break;
	    case '/': buf.setCharAt(i,ZENKAKU_SLASH); break;
	    case '+': buf.setCharAt(i,ZENKAKU_PLUS); break;
	    case '=': buf.setCharAt(i,ZENKAKU_EQUAL); break;
	    case '&': buf.setCharAt(i,ZENKAKU_AND); break;
	    case '|': buf.setCharAt(i,ZENKAKU_VERTBAR); break;
	    case '~': buf.setCharAt(i,ZENKAKU_NAMI); break;
	    case '\u00a5': buf.setCharAt(i,ZENKAKU_YEN); break;
	    }
	}
	
	return buf.toString();
    }
}
